<?php
/**
 * Parser hook extension adds an <asy> tag to wiki markup for rendering
 * Asymptote diagrams within a wiki page.
 * Adapted from MetaUML extension.
 *
 * Parameters:
 *   density=width               passed to ImageMagic to set rendering density.
 *   input=pkg_1,pkg_2,...pkg_n  Asymptote packages to load in prologue.
 *   redraw                      Force redraw.
 *   format                      Target format to use (PNG/SVG), latter is
 *                               not yet implemented here.
 *
 */
 
// Make sure we are being called properly
if( !defined( 'MEDIAWIKI' ) ) {
    echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
    die( -1 );
}
 
class AsymptoteSettings {
    var $asymptoteCommand, $convertCommand;
    var $density, $format, $inputs, $redraw;
 
    function __construct() {
      global $wgImageMagickConvertCommand;
 
      $this->asymptoteCommand = "/usr/bin/asy";
      $this->convertCommand = $wgImageMagickConvertCommand;
      $this->density = "100%";
      $this->format = "png";
      $this->inputs = array();
      $this->redraw = false;
    }
}
 
$wgAsymptoteSettings = new AsymptoteSettings();
 
//Avoid unstubbing $wgParser too early on modern (1.12+) MW versions, as per r35980
if (defined('MW_SUPPORTS_PARSERFIRSTCALLINIT')) {
        $wgHooks['ParserFirstCallInit'][] = 'wfAsymptoteExtension';
} else {
        $wgExtensionFunctions[] = 'wfAsymptoteExtension';
}
 
$wgExtensionCredits['parserhook'][] = array(
        'name' => 'Asymptote',
        'version' => '2.02',
        'author' => 'Andy Hammerlindl, John C. Bowman, Tom Prince',
        'url' => 'http://asymptote.sourceforge.net/',
        'description' => 'Renders an Asymptote diagram with <tt>&lt;asy&gt;...&lt;/asy&gt;</tt> tags.'
);
 
 
function wfAsymptoteExtension() {
  global $wgParser;
  $wgParser->setHook('asy', 'renderAsymptote');
  return true;
}
 
class AsymptoteRenderer {
  var $settings;
  var $src, $hash;
  var $src_path, $ltx_src_path, $dst_path, $dst_url;
  var $old_cwd, $cwd;
 
 function get_hash_path() {
    $path = '/' . $this->hash[0]
          . '/' . $this->hash[1]
          . '/' . $this->hash[2];
    return $path;
  }
 
  function __construct($src) {
    global $wgMathDirectory, $wgMathPath, $wgTmpDirectory;
 
    $this->settings = new AsymptoteSettings;
    $this->old_cwd = getcwd();
    $this->src = $src;
    $this->hash = hash('md5', $src);
 
    wfDebugLog('Asymptote', "Calculated src hash: " . $this->hash);
 
    $dest_dir = $wgMathDirectory . $this->get_hash_path();
 
    if (!is_dir($dest_dir)) {
      mkdir($dest_dir, 0755, true);
    }
 
    if (!is_dir($wgTmpDirectory)) {
      mkdir($wgTmpDirectory, 0755);
    }
 
    $this->src_path = $wgTmpDirectory . '/' . $this->hash;
    $this->ltx_src_path = $wgTmpDirectory . "/ltx-" . $this->hash;
    $this->dst_path = $dest_dir . '/' . $this->hash . '.'
                      . $this->settings->format;
    $this->dst_url = $wgMathPath . $this->get_hash_path() . '/' . $this->hash
                     . '.' . $this->settings->format;
    chdir($wgTmpDirectory);
    wfDebugLog('Asymptote', "src_path: " . $this->src_path);
    wfDebugLog('Asymptote', "dst_path: " . $this->dst_path);
    wfDebugLog('Asymptote', "dst_url: " . $this->dst_url);
  }
 
  function __destruct() {
    $er = error_reporting(E_ALL ^ E_WARNING);
    unlink($this->src_path . ".asy");
    //unlink($this->src_path . ".asyx");
    //unlink($this->ltx_src_path . ".asyx");
    //unlink($this->ltx_src_path . ".tmp");
    unlink($this->ltx_src_path . ".aux");
    unlink($this->ltx_src_path . ".log");
    unlink($this->src_path . ".eps");
    unlink($this->src_path . ".log");
//    unlink($wgTmpDirectory . "/texput.aux");
//    unlink($wgTmpDirectory . "/texput.log");
    chdir($this->old_cwd);
    error_reporting($er);
  }
 
  function create_asymptote() {
    $fd = fopen($this->src_path . ".asy", "wb");
    // fwrite($fd, "prologues := 1;\nnonstopmode;\n\n");
    foreach ($this->settings->inputs as $inp) {
      fwrite($fd, "input " . $inp . ";\n");
    }
    // fwrite($fd, "\nbeginfig(1);\n");
    fwrite($fd, $this->src);
    // fwrite($fd, "\nendfig;\n\nend\n");
    fclose($fd);
  }
 
  function create_image() {
    $asy_cmd = wfEscapeShellArg($this->settings->asymptoteCommand)
                 . " " . $this->src_path . ".asy";
    wfDebugLog('Asymptote', "asy stage: " . $asy_cmd);
    exec($asy_cmd);
    if (!file_exists($this->src_path . ".eps")) {
      return false;
    }
 
    //if (file_exists($this->ltx_src_path . ".tmp")) {
    //    wfDebugLog('Asymptote', "asy stage ltx: " . $asy_cmd);
    //    exec($asy_cmd);
    //}
    //if (!file_exists($this->src_path . ".eps")) {
    //  return false;
    //}
 
    $conv_cmd = wfEscapeShellArg($this->settings->convertCommand)
                . " -density " . wfEscapeShellArg($this->settings->density)
                . " -trim -transparent \"#FFFFFF\" "
                . wfEscapeShellArg($this->src_path . ".eps")
                . " " . wfEscapeShellArg($this->dst_path);
    wfDebugLog('Asymptote', "convert stage: " . $conv_cmd);
    exec($conv_cmd);
    if (!file_exists($this->dst_path)) {
      return false;
    }
 
    return true;
  }
 
  function get_image_url() {
    if (is_file($this->dst_path) && ($this->settings->redraw == true)) {
      unlink($this->dst_path);
    }
 
    if (is_file($this->dst_path)) {
      return $this->dst_url;
    } else {
      $this->create_asymptote();
      if ($this->create_image()) {
        return $this->dst_url;
      } else {
        return false;
      }
    }
  }
}
 
# The callback function for converting the input text to HTML output
function renderAsymptote($src, $argv) {
  $renderer = new AsymptoteRenderer($src);
  global $wgAsymptoteSettings;
 
  if (array_key_exists('density', $argv)) {
    $renderer->settings->density = $argv['density'];
  }
 
  if (array_key_exists('redraw', $argv)) {
    $renderer->settings->redraw = true;
  }
 
  if (array_key_exists('input', $argv)) {
    $c_inp = explode(',', $argv['input']);
    $renderer->settings->inputs = array_unique(array_merge($wgAsymptoteSettings->inputs,
                                                           $c_inp));
  }
 
  if (array_key_exists('format', $argv)) {
    if ($argv['format'] != "png") {
      return "[Formats other than PNG are not yet supported]";
    }
  }
 
  $url = $renderer->get_image_url();
 
  if ($url == false) {
    $text = "[An error occured in Asymptote extension]";
  } else {
    $text = "<img src='$url'>";
  }
 
  return $text;
}

